/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

// IE 8 Fix

$.support.cors = true;

//-----------------------------------------------------------------------------
// iSedora Namespace
//-----------------------------------------------------------------------------

var is = (function() {

  var UA  = navigator.userAgent;
  var IE  = (UA.indexOf('MSIE') > -1 || UA.indexOf('Trident') > -1);
  var IE8 = (IE && UA.indexOf('MSIE 8') > -1);

  // Host Interface

  var host = {
    chooseFile:   function(title) { return isedora('chooseFile', title) },
    chooseFolder: function(title) { return isedora('chooseFolder', title) },
    revealFile:   function(path) { isedora('revealFile', path) },
    openURL:      function(url) { isedora('openURL', url) },
    dndFiles:     function() { return isedora('dndFiles') },
    start:        function() { isedora('start') },
    closeWin:     function() { isedora('closeWin') },
    checkUpdate:  function() { isedora('checkUpdate') },
    getUpdateInterval : function() { return isedora('getUpdateInterval') },
    setUpdateInterval : function(seconds) { isedora('setUpdateInterval', seconds) },
    resizeTo:    function(width, height) { isedora('resizeTo', width, height) },
    openWin:     function(name, url, title, width, height) {
      url = location.protocol + '//' + location.host + url;
      isedora('openWin', name, url, title, width, height);
    },
    newWin:  function(url) {
      if (!url) {
        url = location.href;
      }
      else if (url.substring(0, 7) !== 'http://') {
        url = location.protocol + '//' + location.host + url;
      }
      isedora('newWin', url);
    },
    textGet: function(key) { return isedora('textGet', key) },
    textSet: function(key, val) { isedora('textSet', key, val) },
    textDel: function(key) { isedora('textDel', key) },

    winPropGet: function(key) { return isedora('winPropGet', key) },
    winPropSet: function(key, val) { isedora('winPropSet', key, val) },
    winPropDel: function(key) { isedora('winPropDel', key) }

  };

  // Localization

  function tr(key) {
    var val = trmap[key];
    return (val !== undefined) ? val : key;
  }

  function translate(parent) {
    parent = parent || document;
    $(parent).find("._msg").each(function() {
      var elm = $(this);
      var key = elm.text().trim();
      if (key.length > 0) {
        var val = trmap[key];
        if (val !== undefined && val !== key) {
          elm.text(val);
        }
      }
      elm.removeClass("_msg");
    });
  }

  $(document).ready(function() { translate() });

  // Drag and Drop Prevented by Default

  $(document).ready(function() {
    var html = document.documentElement;
    html.ondrop = function() {
      return false;
    };
    html.ondragover = function() {
      return false;
    };
  });

  // Spinner

  var ispinner = null;

  function spin() {
    if (!ispinner) {
      ispinner = iosOverlay();
    }
    else {
      console.warn('iSpinner is already active');
    }
  }

  function nospin() {
    if (ispinner) {
      ispinner.destroy();
      ispinner = null;
    }
  }

  // Formatting

  function toFixed(num, digits) {
    if (isNaN(num)) return 'NaN';
    var fx = num.toFixed(digits);
    for (;;) {
      var lastChar = fx.charAt(fx.length - 1);
      var llstChar = fx.charAt(fx.length - 2);
      if (lastChar === '0' && llstChar !== '.') {
        fx = fx.substring(0, fx.length - 1);
      }
      else {
        break;
      }
    }
    return fx;
  }

  function toThousand(num) {
    if (typeof num !== 'number' || isNaN(num)) {
      return num;
    }
    return num.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
  }

  function getFileSizeString(size) {
    var den = 1000.0;
    if (size < den) return size + " bytes";
    size /= den;
    if (size < den) return toFixed(size, 2) + " KB";
    size = size / den;
    if (size < den) return toFixed(size, 2) + " MB";
    size = size / den;
    if (size < den) return toFixed(size, 2) + " GB";
    size = size / den;
    return toFixed(size, 2) + " TB";
  }

  function getFileName(fpath) {
    var pos = fpath.lastIndexOf((about.iswin) ? '\\' : '/');
    return (pos > 0 && pos < (fpath.length - 1)) ? fpath.substring(pos + 1) : fpath;
  }

  // Preferences

  function setPref(key, val) {
    if (val === null || val === undefined) {
      localStorage.removeItem(key);
    }
    else {
      localStorage.setItem(key, JSON.stringify(val));
    }
  }

  function getPref(key, def) {
    try {
      var val = localStorage.getItem(key);
      return (val) ? JSON.parse(val) : def;
    } catch (e) {
      //noinspection JSCheckFunctionSignatures
      console.error('Error getting pref %s: %s', key, e.toString());
      return def;
    }
  }

  // Common Pages

  function feedback() {
    host.openWin('feedback', about.face + '/feedback.html', tr('iSedora Feedback'), 500, 600);
  }

  function fileReport(data) {
    var url = about.face + '/feedback.html';
    url += '?title=' + encodeURIComponent('File Report');
    if (data) {
      var hash = md5(data);
      host.textSet(hash, data);
      url += '&data=' + hash;
    }
    host.openWin('feedback', url, tr('File Report'), 500, 600);
  }

  function devReport(data) {
    var url = about.face + '/feedback.html';
    url += '?title=' + encodeURIComponent('Device Report');
    if (data) {
      var hash = md5(data);
      host.textSet(hash, data);
      url += '&data=' + hash;
    }
    host.openWin('feedback', url, tr('Device Report'), 500, 600);
  }

  function credits() {
    host.openWin('credits', about.face + '/credits.html', tr('iSedora Acknowledgements'), 500, 600);
  }

  function openLinkURL(url) {
    host.openURL(url);
    return false;
  }

  function esc(str) {
    return String(str)
      .replace(/&/g, '&amp;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#39;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;');
  }

  // Public Interface

  return {
    isIE: function() {return IE},
    isIE8: function() {return IE8},
    host: host,
    spin: spin,
    nospin: nospin,
    tr: tr,
    msg: tr,
    toFixed: toFixed,
    toThousand: toThousand,
    getFileSizeString: getFileSizeString,
    setPref: setPref,
    getPref: getPref,
    feedback: feedback,
    fileReport: fileReport,
    devReport: devReport,
    credits: credits,
    getFileName: getFileName,
    openLinkURL: openLinkURL,
    esc: esc,
    isMetaDown: function() { return (event.metaKey || (IE && event.ctrlKey)) }
  }


})();

//-----------------------------------------------------------------------------
// Page Helpers
//-----------------------------------------------------------------------------

is.FADE_SPEED = 100;

is.pageout = function(url) {

  var fspeed = is.FADE_SPEED;

  var nwin = (event.type === 'click' && is.isMetaDown());

  if (nwin) {
    console.log('New Win ' + url);
    is.host.newWin(url);
    return;
  }

  if (fspeed === 0) {
    location.href = url;
    return;
  }

  var body = $(document.body);

  $('.navbar').fadeOut(fspeed / 2);

  body.fadeOut(fspeed / 2, function() {
    body.css('overflow', 'hidden');
    window.location = url;
  });

};

$(document).ready(function() {

  var fspeed = is.FADE_SPEED;

  function hookPageouts() {
    if (fspeed === 0) return;
    $(".pageout").each(function() {
      var out = $(this);
      out.removeClass('pageout');
      out.click(function(event) {
        event.preventDefault();
        is.pageout(this.href);
      });
    });

  }

  hookPageouts();

  $('.navbar').fadeIn(fspeed);
  $('body').fadeIn(fspeed);

});

//-----------------------------------------------------------------------------
// Notifications
//-----------------------------------------------------------------------------

is.notify = function(text, type, timeout) {

  if (typeof type !== 'string') type = 'success';
  if (typeof timeout !== 'number') timeout = 5000;

  $.notify(text, {
    autoHide: (timeout > 0),
    autoHideDelay: timeout,
    className: type
  });

};

is.showInfo  = function(text, timeout) { is.notify(text, 'success', timeout) };
is.showWarn  = function(text, timeout) { is.notify(text, 'warn', timeout) };

is.showError = function(text, timeout) {
  if (timeout === undefined) timeout = 0;
  is.notify(text, 'error', timeout)
};

is.notReady  = function() { is.showInfo(is.msg('Not available in this beta version')) };

is.xhrerror = function(xhr) {
  var text = xhr.responseText;
  return (text && text.length) ? text : xhr.statusText;
};

//-----------------------------------------------------------------------------
// Resize Window
//-----------------------------------------------------------------------------

is.resizeWindow = (function() {

  function resize() {
    var defwh = is.getPref("resizeTo", "500x600");
    var wh = prompt("Resize to (WxH):", defwh);
    if (!wh) return;
    var ary = wh.split('x');
    if (ary.length !== 2) return;
    var w = parseInt(ary[0]);
    var h = parseInt(ary[1]);
    if (isNaN(w) || isNaN(h)) return;
    if (is.isIE()) {
      is.host.resizeTo(w, h);
    }
    else {
      window.resizeTo(w, h);
    }
    is.setPref('resizeTo', w + 'x' + h);
  }

  return resize;

})();

//-----------------------------------------------------------------------------
// Page Navigation Shortcust
//-----------------------------------------------------------------------------

$(document).ready(function() {

  var pages = [
    '/face/index.html',
    '/face/prefs.html',
    '/face/minfo.html',
    '/face/about.html'
  ];

  $(document).keydown(function(evt) {
    if (is.isMetaDown()) {
      switch (evt.which) {
        case 49: // 1
        case 50: // 2
        case 51: // 3
        case 52: // 3
          evt.preventDefault();
          var ix = evt.which - 49;
          is.pageout(pages[ix]);
          break;
        case 48: // 0
          evt.preventDefault();
          is.resizeWindow();
          break;
        case 68: // D
          evt.preventDefault();
          is.host.newWin();
          break;
        case 219: // '[' go back
          evt.preventDefault();
          history.back();
          break;
        case 221: // ']' go forward
          evt.preventDefault();
          history.forward();
          break;
      }
    }
  });

});

//-----------------------------------------------------------------------------
// jQuery Plugins
//-----------------------------------------------------------------------------

$.fn.disable = function() {
  return this.each(function() {
    $(this).attr('disabled', 'disabled');
  });
};

$.fn.enable = function(flag) {
  if (flag === undefined) flag = true;
  return this.each(function() {
    if (flag) {
      $(this).removeAttr('disabled');
    }
    else {
      $(this).attr('disabled', 'disabled');
    }
  });
};
