/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Inspector

var mi = (function() {

  var EMPTY_IMAGE = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7';

  var fadeSpeed = is.FADE_SPEED;

//  var retina = ()

  var busy = false;
  var data = null;
  var mode = 0;
  var dstart;
  var tstart;
  var currentPath;
  var currentIndex;

  var tr = is.tr;

  var spinner = new Spinner({
    lines:   13,
    length:  7,
    width:   3,
    radius:  8,
    corners: 1,
    rotate:  0,
    hwaccel: false,
    top: '150px'
  });

  function hide(jqe, cb) {
    jqe.fadeOut(fadeSpeed, cb)
  }

  function show(jqe, cb) {
    jqe.fadeIn(fadeSpeed, cb);
  }

  function spin() {
    spinner.spin(document.body);
  }

  function nospin() {
    spinner.stop();
    busy = false;
    $('.onbusy').enable();
  }

  //-----------------------------------------------------------------------------
  // History
  //-----------------------------------------------------------------------------

  function encodePathURL(fpath, index) {
    fpath += ';' + index;
    return "#" + encodeURIComponent(fpath);
  }

  function remember(fpath, index) {
    window.location = encodePathURL(fpath, index);
  }

  $(document).ready(function() {
    var hash = window.location.hash;
    if (hash && hash.length > 1) {
      $('#mi-start').hide();
      var fpath = decodeURIComponent(hash.substring(1));
      var index = 0;
      var ipos = fpath.lastIndexOf(';');
      if (ipos > 0 && ipos < (fpath.length - 1)) {
        try {
          index = parseInt(fpath.substring(ipos + 1));
          fpath = fpath.substring(0, ipos);
        } catch (e) {
          index = 0;
        }
      }
      loadFile(fpath, index);
    }
  });

  // === RTI ===

  var id    = rti.id;
  var ftype = rti.ftype;
  var mtype = rti.mtype;

  // Media Type Names

  var mtname = {};

  mtname[mtype.image] = is.tr('Image');
  mtname[mtype.audio] = is.tr('Audio');
  mtname[mtype.video] = is.tr('Video');
  mtname[mtype.pdf]   = is.tr('PDF document');

  // Sort Oroder

  var vsort = [
    id.error,
    id.mtype,
    id.flink,
    id.fmime,
    id.title,
    id.count,
    id.width,
    id.orn,
    id.bpp,
    id.clr,
    id.fsize,
    id.ctime,
    id.mtime,
    id.fhidn
  ];

  // Value Mappers

  var vmap = {};

  vmap[id.flink] = function(src, val, dst) {
    var link = $('<a href="#"/>');
    link.click(function(evt) {
      evt.preventDefault();
      is.host.revealFile(val);
    });
    link.text(val);
    dst.push(['Link To', link])
  };

  vmap[id.fhidn] = function(src, val, dst) {
    if (val) {
      dst.push(['Hidden', 'Yes']);
    }
  };

//  vmap[id.ctime] = function(src, val, dst) {
//    var d = new Date(val);
//    dst.push(['Created', d.toLocaleString()]);
//  };
//
//  vmap[id.mtime] = function(src, val, dst) {
//    var d = new Date(val);
//    dst.push(['Modified', d.toLocaleString()]);
//  };

  vmap[id.fmime] = function(src, val, dst) {dst.push(['Mime Type', val])};

  vmap[id.fsize] = function(src, val, dst) {dst.push(['File Size', is.getFileSizeString(val)])};
//  vmap[id.ftype] = function(src, val, dst) {
//    if (val === ftype.folder) {
//      dst.push(['Path Type', is.tr('Folder')])
//    }
//    else if (val === ftype.unknown) {
//      dst.push(['Path Type', is.tr('Unknown Path Type')])
//    }
//  };

  vmap[id.error] = function(src, val, dst) {
    var text = $('<span style="color: darkred;"/>');
    text.text(is.tr(val));
    dst.push(['Error', text]);
  };

  vmap[id.mtype] = function(src, val, dst) {
    var text;
    if (rti.isFolder(src)) {
      text = is.tr('Folder');
    }
    else if (val === mtype.unknown) {
      text = $('<span style="color: darkred;"/>');
      text.text(is.tr('Unknown'));
    }
    else {
      text = mtname[val];
    }
    dst.push(['Media Type', text]);
  };

  vmap[id.title] = function(src, val, dst) {
    dst.push(['Title', val]);
  };

  vmap[id.width] = function(src, val, dst) {
    var hgt = src[id.height];
    var wtext = (val > -1) ? is.toThousand(val) : is.tr('auto');
    var htext = (hgt > -1) ? is.toThousand(hgt) : is.tr('auto');
    dst.push(['Width / Height', wtext + ' x ' +  htext]);
  };

  vmap[id.bpp] = function(src, val, dst) {
    dst.push(['Bits per Pixel', val]);
  };

  vmap[id.clr] = function(src, val, dst) {
    if (src[id.idxclr]) val += ' (Indexed Color)';
    dst.push(['Color Model', val]);
  };

  vmap[id.count] = function(src, val, dst) {
    if (rti.isPDF(src)) {
      dst.push(['Page', (src[id.index] + 1) + ' of ' + val]);
    }
  };

  vmap[id.orn] = function(src, val, dst) {
    var otext;
    switch (val) {
      case 0:
      case 1:
        return;
      case 2:
        otext = is.tr("Mirror horizontal");
        break;
      case 3:
        otext = is.tr("Rotate 180");
        break;
      case 4:
        otext = is.tr("Mirror vertical");
        break;
      case 5:
        otext = is.tr("Mirror horizontal and rotate 270 CW");
        break;
      case 6:
        otext = is.tr("Rotate 90 CW");
        break;
      case 7:
        otext = is.tr("Mirror horizontal and rotate 90 CW");
        break;
      case 8:
        otext = is.tr("Rotate 270 CW");
        break;
      default:
        return;
    }
    dst.push(['Orientation', otext]);
  };

  // File Report Filter

  var vfilter = [id.uid, id.fpath, id.ctime, id.mtime, id.flink];

  function filterReport(mdata, report) {
    for (var key in mdata) {
      if (vfilter.indexOf(key) > -1) {
        continue;
      }
      var val = mdata[key];
      if (rti.isObject(val)) {
        var subobj = {};
        filterReport(val, subobj);
        report[key] = subobj;
      }
      else {
        report[key] = val;
      }
    }
  }

  // Sort Order

  function valueArray(mdata) {
    var ary = [];
    for (var key in mdata) {
      var val = mdata[key];
      if (rti.isObject(val)) {
        ary.push([key, {sub: valueArray(val), val: val}]);
      }
      else {
        ary.push([key, val]);
      }
    }
    ary.sort(function(v1, v2) {
      var x1 = vsort.indexOf(v1[0]);
      var x2 = vsort.indexOf(v2[0]);
      if (x1 < 0) x1 = 1000;
      if (x2 < 0) x2 = 1000;
      return x1 - x2;
    });
    return ary;
  }

  // Map Values

  function mapValues(mdata, srcary, dstary) {
    for (var i = 0; i < srcary.length; i++) {
      var kv = srcary[i];
      var key = kv[0];
      var val = kv[1];
      if (rti.isObject(val)) {
        var subary = [];
        mapValues(val.val, val.sub, subary);
        if (subary.length) {
          dstary.push([key, {sub: subary}]);
        }
      }
      else {
        var vm = vmap[key];
        if (vm) {
          vm(mdata, val, dstary);
        }
      }
    }
  }

  //-----------------------------------------------------------------------------
  // Thumb handling
  //-----------------------------------------------------------------------------

  function onThumbLoad(img) {
    if (img.src === EMPTY_IMAGE) return;
    var tend = Date.now();
    nospin();
    show($('#mi-thumb'));
    var ttime = is.toFixed((tend - tstart) / 1000.0, 3);
    data['ttime'] = ttime;
    $('#mi-wtime').append(' | ' + is.tr('Thumb Time') + ': ' + ttime);
  }

  function onThumbFail(img) {
    img.src = EMPTY_IMAGE;
    hide($('#mi-thumb'));
    nospin();
    is.showError(is.tr('Error loading thumbnail'), 2000);
  }

  function loadThumb(md) {
    var tmb = $('#mi-thumb');
    tmb.attr('title', 'WxH: ' + md[id.twidth] + 'x' + md[id.theight]);
    var src = about.dlna + '/thumb/' + md[id.uid] + '.jpg' + '?nocache=' + Math.random();
    tstart = Date.now();
    tmb.attr('src', src);
  }

  //-----------------------------------------------------------------------------
  // Pagination
  //-----------------------------------------------------------------------------

  function makePages(md, fpath, index) {
    var pages = $('#mi-pages');
    pages.empty();
    var count = md[id.count];
    if (count == 0) {
      return;
    }
    if (index > 0) {
      var prev = $('<a href="#">&larr;&nbsp;</a>');
      prev.append(is.tr('Prev Page'));
      prev.click(function(evt) {
        evt.preventDefault();
        loadFile(fpath, index - 1);
      });
      pages.append(prev);
    }
    if (index < (count - 1)) {
      if (index > 0) {
        pages.append('&nbsp;|&nbsp;');
      }
      var next = $('<a href="#">&nbsp;&rarr;</a>');
      next.prepend(is.tr('Next Page'));
      next.click(function(evt) {
        evt.preventDefault();
        loadFile(fpath, index + 1);
      });
      pages.append(next);
    }
  }

  //-----------------------------------------------------------------------------
  // WorkFlow
  //-----------------------------------------------------------------------------

  function onData(fpath, index, mdata) {

    var finish = Date.now();

    currentPath  = fpath;
    currentIndex = index;

    remember(fpath, index);

    loadThumb(mdata);

    var tbl = $('#mi-table');

    tbl.empty();

    var fname = is.getFileName(fpath);

    var flink = $('<a href="#"/>');

    flink.text(fname);
    flink.attr('title', is.tr('Show in Folder'));

    flink.click(function(evt) {
      evt.preventDefault();
      is.host.revealFile(fpath);
    });

    tbl.append($('<tr/>').append($('<th colspan="2"/>').append(flink)));


    var srcary = valueArray(mdata);
    var dstary = [];

    mapValues(mdata, srcary, dstary);

    for (var i = 0; i < dstary.length; i++) {
      var kv = dstary[i];
      var row = $('<tr/>');
      row.append($('<td class="c1"/>').html(tr(kv[0]) + ':'));
      row.append($('<td class="c2"/>').html(kv[1]));
      tbl.append(row);
    }

    data = mdata;

    var wtime = is.toFixed((finish - dstart) / 1000.0, 3);

    data['wtime'] = wtime;

    $('#mi-wtime').text(is.tr('Work Time') + ': ' + wtime);

    makePages(mdata, fpath, index);

    show($('#mi-data'));

  }

  //-----------------------------------------------------------------------------
  // Load File
  //-----------------------------------------------------------------------------

  function loadFile(fpath, index) {

    busy = true;

    $('.onbusy').disable();

    function loader() {
      spin();
      dstart = Date.now();
      $.ajax({
          url: about.rest + '/minfo',
          data: {fpath: fpath, index: index},
          dataType: 'json',
          cache: false
        }
      )
        .done(function(mdata) {
          mode = 1;
          onData(fpath, index, mdata);
        })
        .fail(function(xhr) {
          nospin();
          window.location = '#';
          is.showError(is.xhrerror(xhr), 5000);
          show($('#mi-start'));
          mode = 0;
        });
    }

    if (mode) {
      hide($('#mi-thumb'));
      hide($('#mi-data'), loader);
    }
    else {
      hide($('#mi-start'), loader);
    }

  }

  // Choose File

  function chooseFile(chooseMode) {

    var folder = (window.event && window.event.altKey);

    var fpath;

    if (!folder) {
      fpath = is.host.chooseFile(is.tr("Open Media File"));
    }
    else {
      fpath = is.host.chooseFolder(is.tr("Open Media Folder"));
    }

    if (fpath) {
      mode = chooseMode;
      loadFile(fpath, 0)
    }

  }

  // Key Binding

  $(document).keydown(function(evt) {
    if (!busy && evt.keyCode == 79) {
      chooseFile(mode);
      evt.preventDefault();
      evt.stopPropagation();
    }
  });

  // Drang and Drop Binding

  $(document).ready(function() {

    var html = document.documentElement;

    var dndCount = 0;

    function checkDND() {
      var fpaths = is.host.dndFiles();
      if (Array.isArray(fpaths) && fpaths.length > 0) {
        loadFile(fpaths[0], 0);
      }
      else if (dndCount++ < 3) {
        setTimeout(checkDND, 100);
      }
    }

    html.ondrop = function() {
//      console.log('Drop()');
      dndCount = 0;
      if (!busy) {
        setTimeout(checkDND, 100);
      }
      return !busy && is.isIE();
    };

    html.ondragover = function() {
//      console.log('DragOver()');
      event.dataTransfer.dropEffect = (busy) ? 'none' : 'copy';
      return false;
    };

  });

  // Clear

  function clear() {
    mode = 0;
    data = null;
    currentPath = null;
    currentIndex = 0;
    hide($('#mi-thumb'));
    hide($('#mi-data'), function() {
      window.location = '#';
      show($('#mi-start'));
      $('#mi-table').empty();
      $('#mi-thumb').attr('src', EMPTY_IMAGE);
    });
  }

  // Send Report

  function sendReport() {
    var rep = {};
    filterReport(data, rep);
    rep = JSON.stringify(rep, null, 2);
    rep = rep.replace(/"/g, '');
    is.fileReport(rep);
  }

  // Refresh

  function refresh() {
    loadFile(currentPath, currentIndex);
  }

  // Public Interface

  return {
    chooseFile: chooseFile,
    clear: clear,
    refresh: refresh,
    sendReport: sendReport,
    thumbLoad: onThumbLoad,
    thumbFail: onThumbFail
  }

})();