/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var express = require('express');

var evt = require("./evt");
var env = require("./env");
var log = require("./log");
var cfg = require("./cfg");
var acl = require("./acl");
var HTTPServer = require('./cls/httpsrv');

var ROUTER_OPTS = { caseSensitive: true, strict: false };

var srv = null;

//-----------------------------------------------------------------------------

function makeRouter(app, path) {
  var r = express.Router(ROUTER_OPTS);
  app.use(path, r);
  app.set(path, r);
  return r;
}

//-----------------------------------------------------------------------------

function start() {

  if (srv) {
    throw "Server Already Started";
  }

  var app = express();

  app.set('env', 'production');
  app.enable('case sensitive routing');
  app.disable('x-powered-by');
  app.set('etag', false);

  var rest = makeRouter(app, cfg.restPath());
  var dlna = makeRouter(app, cfg.dlnaPath());
  var face = makeRouter(app, cfg.facePath());

  rest.use(acl.adminAuth);
  face.use(acl.adminAuth);
  dlna.use(acl.dlnaAuth);

  evt.postHttpAppReady(app);

  app.use(function(err, req, res, next) {
    var code = err.code;
    if (code) {
      switch (code) {
        case 'EPIPE':
        case 'ECONNABORTED':
          return;
      }
    }
    log.error('Request error: %s\n%s', req.originalUrl, env.estack(err));
    if (!res.headersSent) {
      res.status(500).send(err.toString());
    }
  });

  srv = new HTTPServer();

  srv.on('close', function() {});

  srv.on('error', function(err) {
    log.error('HTTP Server error: %s', err);
    evt.throwGlobal(err);
  });

  srv.on('ready', function(addr) {
    cfg.setHttpPort(addr.port);
    evt.postHttpSrvReady(addr);
  });

  srv.start(app, cfg.httpPort());

}

exports.start = start;



