/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var fs = require('fs');
var np = require('path');

var WOPTS = { persistent: false };

function FileWatcher(changecb) {

  var self  = this;
  var watch = null;
  var fpath = null;

  function stopWatch() {
    if (watch) {
      try { watch.close() } catch (e) {}
      watch = null;
    }
  }

  function startWatch(filePath) {
    fpath = filePath;
    watch = fs.watch(fpath, WOPTS);
    watch.once('change', onChange);
    watch.once('error', onError);
  }

  function onError(err) {
    stopWatch();
    changecb(err);
  }

  function onChange() {
    stopWatch();
    changecb(null, self);
  }

  self.start = function(filePath) {
    stopWatch();
    startWatch(filePath);
  };

  self.restart = function() {
    if (fpath) {
      stopWatch();
      startWatch(fpath);
    }
  };

  self.fpath = function() { return fpath };

  self.dirname = function() { return (fpath) ? np.dirname(fpath) : null };

  self.stop  = function() {
    stopWatch();
    fpath = null;
  };

  self.isWatching = function() { return (watch !== null) };

}

exports.FileWatcher = FileWatcher;