/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Handler Requests

'use strict';

var rti  = require('../rti');
var didl = require('./didl');
var util = require('util');

var did = didl.id;
var mtype = rti.mtype;

//-----------------------------------------------------------------------------

function BaseRequest(dev) {
  this.dev = dev;
}

//-----------------------------------------------------------------------------

function BrowseRequest(dev, oid, req, meta, furl, turl, start, count) {

  BaseRequest.call(this, dev);

  this.oid = oid;
  this.req = req;
  this.uid = null;
  this.ary = [];
  this.mt = 0;
  this.meta = meta;
  this.furl = furl;
  this.turl = turl;

  this.start = start;
  this.count = count;
  this.range = false;
  this.total = 0;

}

util.inherits(BrowseRequest, BaseRequest);

//-----------------------------------------------------------------------------

BrowseRequest.prototype.append = function(item) {
  this.ary.push(item);
};

BrowseRequest.prototype.addall = function(items) {
  var len = items.length;
  for (var i = 0; i < len; i++) {
    var item = items[i];
    if (item !== null) {
      this.ary.push(item);
    }
  }
};

BrowseRequest.prototype.prepend = function(item) {
  this.ary.unshift(item);
};

BrowseRequest.prototype.insert = function(index, item) {
  this.ary.splice(index, 0, item);
};

BrowseRequest.prototype.item = function(index) {
  return this.ary[index];
};

BrowseRequest.prototype.indexOfMediaClass = function(mediaClass) {
  var len = this.ary.length;
  for (var i = 0; i < len; i++) {
    var item = this.ary[i];
    if (item[did.mediaClass] === mediaClass) {
      return i;
    }
  }
  return -1;
};

BrowseRequest.prototype.isImage = function() {
  return (this.mt === mtype.image);
};

BrowseRequest.prototype.isAudio = function() {
  return (this.mt === mtype.audio);
};

BrowseRequest.prototype.isVideo = function() {
  return (this.mt === mtype.video);
};

Object.defineProperty(BrowseRequest.prototype, 'length', {
  get: function() { return this.ary.length }
});

exports.BrowseRequest = BrowseRequest;

//-----------------------------------------------------------------------------
// Stream Request
//-----------------------------------------------------------------------------

function StreamRequest(dev, req, res, oid, prf, tmb) {

  BaseRequest.call(this, dev);

  this.req = req;
  this.res = res;
  this.oid = oid;
  this.prf = prf;
  this.tmb = tmb;
  this.mt  = 0;
  var pos = oid.indexOf('_');
  if (pos > 0) {
    this.hnd = oid.substring(0, pos);
    this.uid = oid.substring(pos + 1);
  }
  else {
    this.hnd = null;
    this.uid = null;
  }
}

util.inherits(StreamRequest, BaseRequest);

//-----------------------------------------------------------------------------

StreamRequest.prototype.isImage = function() {
  return (this.mt === mtype.image);
};

StreamRequest.prototype.isAudio = function() {
  return (this.mt === mtype.audio);
};

StreamRequest.prototype.isVideo = function() {
  return (this.mt === mtype.video);
};

exports.StreamRequest = StreamRequest;