/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var async = require('async');

var evt = require('../evt');
var ifa = require('../ifa');
var acl = require('../acl');
var log = require('../log');
var cfg = require('../cfg');

var ssdp = require('./ssdp');

var START_TIMEOUT = 2000; // 3 secs

function Notify(udn, location, httpPort, usnary) {

  var self = this;
  var tout = null; // timeout
  var done = false;

  self.start = function() {
    log.debug('Notify started');
    evt.onSsdpMessage(onSsdpMessage);
    evt.onNetFaceOn(onFaceOn);
    evt.onConfigChange(onConfigChange);
    tout = setTimeout(repeatAlive, START_TIMEOUT);
  };

  self.stop = function(cb) {
    done = true;
    resetTimer();
    var fary = acl.filterDlnaFaces(ifa.sysFaces());
    async.each(fary, byebye, cb);
  };

  self.notifyAll = function(cb) {
    var fary = acl.filterDlnaFaces(ifa.sysFaces());
    async.each(fary, alive, cb);
  };

  self.notifyByeBye = function(cb) {
    var fary = acl.filterDlnaFaces(ifa.sysFaces());
    async.each(fary, byebye, cb);
  };

  function repeatAlive() {
    var fary = acl.filterDlnaFaces(ifa.sysFaces());
    async.each(fary, alive);
    tout = setTimeout(repeatAlive, cfg.aliveInerval() * 1000);
  }

  function alive(face, cb) {
    notify(face, ssdp.ALIVE, cb);
  }

  function byebye(face, cb) {
    notify(face, ssdp.BYEBYE, cb);
  }

  function update(face, cb) {
    notify(face, ssdp.UPDATE, cb);
  }

  function notify(face, verb, cb) {
    ssdp.notify(face, udn, makeLocation(face), usnary, verb, cb);
  }

  function makeLocation(face) {
    return 'http://' + face.addr + ':' + httpPort + location;
  }

  function resetTimer() {
    if (tout) {
      clearTimeout(tout);
      tout = null;
    }
  }

  function onFaceOn(face) {
    if (acl.checkFace(face)) {
      setTimeout(function() {
        alive(face);
      }, START_TIMEOUT);
    }
  }

  function onConfigChange() {
    resetTimer();
    var fary = ifa.sysFaces();
    async.each(fary, byebye, function() {
      tout = setTimeout(repeatAlive, 1000);
    });
  }

  //-----------------------------------------------------------------------------
  // Search Response
  //-----------------------------------------------------------------------------

  function onSsdpMessage(msg) {
    if (done || !msg.isSearch()) return;
    var st = msg.st;
    var uary;
    if (st === ssdp.SSDP_ALL) {
      uary = usnary;
    }
    else if (usnary.indexOf(st) > -1) {
      uary = [st];
    }
    else {
      return;
    }
    var mx = (msg.mx * 1000) - 100;
    mx = Math.floor(Math.random() * mx) + 1;
    setTimeout(function() {
      ssdp.searchResponse(msg, udn, makeLocation(msg.face), uary);
    }, mx);
  }

}

exports.Notify = Notify;