/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var SLOT_KEYS = '0123456789qwertyuiopasdfghjklzxcvbnm';

function SlotMap() {

  var self = this;
  var data = {};

  //-----------------------------------------------------------------------------

  function ctor() {
    for (var i = 0; i < SLOT_KEYS.length; i++) {
      slot(SLOT_KEYS.charAt(i));
    }
  }

  ctor();

  //-----------------------------------------------------------------------------

  function slot(key) {
    var k = key.charAt(0);
    var s = data[k];
    if (s === undefined) {
      s = {};
      data[k] = s;
    }
    return s;
  }

  //-----------------------------------------------------------------------------

  function get(key) {
    return slot(key)[key];
  }

  function set(key, val) {
    slot(key)[key] = val;
  }

  function has(key) {
    return (slot(key)[key] !== undefined);
  }

  function del(key) {
    var s = slot(key);
    delete s[key];
  }

  //-----------------------------------------------------------------------------

  function each(cb) {
    for (var k in data) {
      var s = data[k];
      for (var key in s) {
        if (cb(key, s[key]) === false) {
          return;
        }
      }
    }
  }

  function clear() {
    for (var k in data) {
      data[k] = {};
    }
  }

  function allkeys() {
    var keys = [];
    for (var k in data) {
      var s = data[k];
      for (var key in s) {
        keys.push(key);
      }
    }
    return keys;
  }

  //-----------------------------------------------------------------------------

  self.get = get;
  self.set = set;
  self.has = has;
  self.del = del;

  self.each    = each;
  self.clear   = clear;
  self.allkeys = allkeys;

}

exports.SlotMap = SlotMap;