/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Image File Streamer

'use strict';

var evt = require('../evt');
var hnd = require('../hnd');
var log = require('../log');
var rti = require('../rti');
var htp = require('../htp');
var cfg = require('../cfg');
var mdb = require('./mdb');
var tmb = require('./tmb');
var alg = require('./alg');
var prf = require('./prf');
var didl = require('../cls/didl');

var Res = didl.Res;

var id = rti.id;
var mtype = rti.mtype;

var minfo = mdb.minfo;
var scale = mdb.scale;

var downscale;
var autorotate;

//-----------------------------------------------------------------------------
// Browse Handlers
//-----------------------------------------------------------------------------

function imageItem(brq, md, cb) {
  if (!rti.isImage(md)) return;
  cb(null, makeImageItem(brq, md));
  return true;
}

hnd.addMediaHandler(imageItem);

//-----------------------------------------------------------------------------
// Image Item
//-----------------------------------------------------------------------------

function makeImageItem(brq, md) {
  var title = md[id.fname];
  if (!cfg.hideExt()) {
    var ext = md[id.fextn];
    if (ext) {
      title += '.' + ext;
    }
  }
  var iid  = hnd.fsoid(mtype.image, rti.uid(md));
  var img = didl.imageItem(iid, brq.oid, title);
  img.setDate(md[id.mtime]);
  var tc = tcodeParams(brq.dev, md);
  var iprf = resolveImageProfile(tc.w, tc.h);
  var ires = new Res(hnd.streamURL(brq.furl, iid, iprf), iprf);
  ires.setColorDepth(24);
  ires.setResolution(tc.w, tc.h);
  img.addResource(ires);
  if (md[id.tvalid]) {
    var tres = new Res(hnd.streamURL(brq.turl, iid, prf.JPEG_TN), prf.JPEG_TN);
    tres.setColorDepth(24);
    tres.setResolution(md[id.twidth], md[id.theight]);
    img.addResource(tres);
  }
  return img;
}

exports.makeImageItem = makeImageItem;

//-----------------------------------------------------------------------------
// Stream Handlers
//-----------------------------------------------------------------------------

function imageStream(srq, imagePath) {
  htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
  evt.postStartStream();
  srq.res.sendFile(imagePath, htp.SEND_FILE_OPT, evt.postStopStream);
}

exports.imageStream = imageStream;

//-----------------------------------------------------------------------------

function thumbStream(srq, md) {
  htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
  tmb.sendThumb(md, srq.req, srq.res);
  return true;
}

function imageStreamHandler(srq) {
  if (srq.mt !== mtype.image || srq.hnd !== hnd.FILE) return;
  var uid = srq.uid;
  var md = minfo.get(uid);
  if (!md) return;
  if (srq.tmb) return thumbStream(srq, md);
  if (!rti.isImage(md)) return;
  var tc = tcodeParams(srq.dev, md);
  if (!tc.tcode) {
    imageStream(srq, rti.fpath(md));
  }
  else {
    htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
    tmb.sendScale(md, tc.w, tc.h, srq.res);
  }
  return true;
}

hnd.addStreamHandler(imageStreamHandler);

//-----------------------------------------------------------------------------

function resolveImageProfile(w, h) {
//  if (w <= 160 && h <= 160)  return prf.JPEG_TN;
//  if (w <= 640 && h <= 480)  return prf.JPEG_SM;
//  if (w <= 1024 && h <= 768) return prf.JPEG_MED;
  return prf.JPEG_LRG;
}

//-----------------------------------------------------------------------------

function tcodeParams(dev, md) {
  var flag = false;
  var w = md[id.width];
  var h = md[id.height];
  var sr = dev.sreso;
  if ((w < 0 || h < 0) || (downscale && (w > downscale[0] || h > downscale[1]) && (w > sr[0] || h > sr[1]))) {
    flag = true;
    var wh = alg.scale(w, h, sr[0], sr[1]);
    w = wh[0];
    h = wh[1];
  }
  if (!flag && autorotate && md[id.orn] > 4) {
    flag = true;
  }
  if (!flag && md[id.fmime] !== 'image/jpeg') {
    flag = true;
  }
  if (!flag && md[id.bpp] !== 24) {
    flag = true;
  }
  return { tcode: flag, w: w, h: h };
}

//-----------------------------------------------------------------------------

function onConfig() {
  autorotate = cfg.imageRotate();
  downscale = cfg.imageScale();
  if (downscale === 'none') {
    downscale = null;
  }
  else {
    var wh = downscale.split('x');
    downscale = [parseInt(wh[0], 10), parseInt(wh[1], 10)];
  }
}

evt.onConfigReadyChange(onConfig);