/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Database

'use strict';

var evt = require("../evt");
var cfg = require("../cfg");
var rti = require("../rti");
var jsl = require("../jsl");

var hasPrefix = jsl.hasPrefix;

var cache = require('../cls/cache');

var THUMB_EXT = '.jpg';
var SCALE_EXT = '.jpg';

var thumbCache = new cache.SizeCache('thumb', THUMB_EXT, 32);
var scaleCache = new cache.SizeCache('scale', SCALE_EXT, 32);
var minfoCache = new cache.JsonFileCache('minfo', '.json', 32);
var parents    = new cache.KeyValStore('parents', 3000, true);

exports.thumb   = thumbCache;
exports.minfo   = minfoCache;
exports.scale   = scaleCache;
exports.parents = parents;

evt.onConfigReady(function() {
  thumbCache.load(cfg.thumbCache());
  scaleCache.load(cfg.scaleCache());
  minfoCache.load(cfg.minfoCache());
  parents.load(cfg.cacheDir(), 'parents.json');
});



