/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Info Handling

'use strict';

var fs = require('fs');
var np = require('path');
//var dash = require('lodash');
var mime = require('mime');
var log = require("../log");
var agt = require("../agt");
var env = require("../env");
var md5 = require("../md5");
var cfg = require("../cfg");
var mdb = require("./mdb");
var alg = require("./alg");
var rti = require("../rti");
var fop = require("../fop");
var evt = require("../evt");

var id    = rti.id;
var ftype = rti.ftype;
var mtype = rti.mtype;

var minfo = mdb.minfo;

//var PDF_MIME   = 'application/pdf';
var BIN_MIME   = 'application/octet-stream';

var THUMB_DLNA  = 160;
var THUMB_WINXP = 48;
//var RETINA_EXT = '@2x';

//-----------------------------------------------------------------------------

function emptyData(uid, fpath) {
  var md = {};
  md[id.uid]   = uid;
  md[id.fpath] = fpath;
  md[id.ftype] = ftype.unknown;
  md[id.mtype] = mtype.unknown;
  setThumbDimension(md);
  return md;
}

//-----------------------------------------------------------------------------

//function mediaStat(filePath, cb) {
//  fs.stat(filePath, function(err, stat) {
//    if (err) {
//      return cb(err);
//    }
//    var md = {};
//    md[id.fpath] = filePath;
//    md[id.ftype] = ftype.unknown;
//    md[id.mtype] = mtype.unknown;
//    md[id.ctime] = stat.ctime.getTime();
//    md[id.mtime] = stat.mtime.getTime();
//    md[id.fmode] = stat.mode || 0;
//    md[id.fhidn] = false;
//    if (stat.isFile()) {
//      md[id.ftype] = ftype.file;
//      md[id.fsize] = stat.size;
//      var fname = path.basename(filePath);
//      var fextn = null;
//      var expos = fname.lastIndexOf('.');
//      if (expos > 0 && expos < (fname.length - 1)) {
//        fextn = fname.substring(expos + 1).toLowerCase();
//        fname = fname.substring(0, expos);
//        md[id.fmime] = mime.lookup(fextn);
//      }
//      else {
//        md[id.fmime] = BIN_MIME;
//      }
//      md[id.fname] = fname;
//      md[id.fextn] = fextn;
//    }
//    else if (stat.isDirectory()) {
//      md[id.ftype] = ftype.folder;
//      md[id.fname] = path.basename(filePath);
//    }
//    if (md[id.ftype] > 0) {
//      md[id.fhidn] = (md[id.fname].charAt(0) === '.');
//    }
//    cb(null, md);
//  });
//}

//-----------------------------------------------------------------------------

function fileInfo(fpath, index, cache, cb) {

  var uid = md5(fpath);

  if (index > 0) uid += '_' + index;

  if (cache) {
    var md = minfo.get(uid);
    if (md) {
      return cb(null, md);
    }
  }

  agt.minfo(fpath, index, function(err, md) {

    if (err) {
      log.error('minfo error: %s', err);
      var zmd = emptyData(uid, fpath);
      zmd[id.error] = err.toString();
      minfo.set(uid, zmd);
      return cb(null, zmd);
    }

    md[id.uid] = uid;

    setThumbDimension(md);

    if (md[id.ftype] == ftype.unknown) {
      minfo.set(uid, md);
      return cb(null, md);
    }

    if (rti.isFile(md)) {
      if (md[id.fextn]) {
        md[id.fmime] = mime.lookup(md[id.fextn]);
      }
      else {
        md[id.fmime] = BIN_MIME;
      }
    }

    md[id.stamp] = Date.now();

    minfo.set(uid, md);

    cb(null, md);

  });

}

exports.fileInfo = fileInfo;

//-----------------------------------------------------------------------------

function setThumbDimension(md) {

  md[id.tvalid] = true;

  if (rti.isImageOrPDF(md)) {
    var w = md[id.width];
    var h = md[id.height];
    if (w > 0 && h > 0) {
      var tw = THUMB_DLNA;
      var th = THUMB_DLNA;
      if (w > tw || h > th) {
        var sz = alg.scale(w, h, tw, th);
        tw = sz[0];
        th = sz[1];
      }
      else {
        tw = w;
        th = h;
      }
      md[id.twidth] = tw;
      md[id.theight] = th;
    }
    else {
      md[id.twidth]  = THUMB_DLNA;
      md[id.theight] = THUMB_DLNA;
    }
  }
  else {
    var tsize = THUMB_DLNA;
    md[id.twidth]  = tsize;
    md[id.theight] = tsize;
  }

}



