/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// DLNA Profiles

'use strict';

var evt = require('../evt');
var flags = require('./flg');

var id = {
  name: 'name',
  mime: 'mime',
  flag: 'flag'
};

exports.id = id;

function Profile(name, mime, flag) {
  this[id.name] = name;
  this[id.mime] = mime;
  this[id.flag] = flag;
}

Profile.prototype.toString = function(short, conversion, operation) {
  var rv = (short) ? '' : 'http-get:*:' + this[id.mime] + ':';
  rv += 'DLNA.ORG_PN=' + this[id.name] + ';';
  if (conversion !== undefined && conversion !== null) {
    rv += 'DLNA.ORG_CI=' + conversion + ';';
  }
  if (operation !== undefined && operation !== null) {
    rv += 'DLNA.ORG_OP=' + operation + ';';
  }
  rv += 'DLNA.ORG_FLAGS=' + this[id.flag];
  return rv;
};

exports.Profile = Profile;

function Image(name, mime) {
  return new Profile(name, mime, flags.PHOTO);
}

function Audio(name, mime) {
  return new Profile(name, mime, flags.MUSIC);
}

function Video(name, mime) {
  return new Profile(name, mime, flags.VIDEO);
}

var JPEG = 'image/jpeg';
var PNG  = 'image/png';
var BMP  = 'image/x-ms-bmp';
var MP3  = 'audio/mpeg';
var LPCM = 'audio/L16';
var FLAC = 'audio/x-flac';
var WMA  = 'audio/x-ms-wma';
var WAV  = 'audio/wav';
var MPEG = 'video/mpeg';
var MTTS = 'video/vnd.dlna.mpeg-tts';
var WMV  = 'video/x-ms-wmv';

var JPEG_TN  = Image('JPEG_TN', JPEG);
var JPEG_SM  = Image('JPEG_SM', JPEG);
var JPEG_MED = Image('JPEG_MED', JPEG);
var JPEG_LRG = Image('JPEG_LRG', JPEG);

exports.JPEG_TN  = JPEG_TN;
exports.JPEG_SM  = JPEG_SM;
exports.JPEG_MED = JPEG_MED;
exports.JPEG_LRG = JPEG_LRG;

var all = [

  JPEG_TN,
  JPEG_SM,
  JPEG_MED,
  JPEG_LRG,

  Image('PNG_TN', PNG),
  Image('PNG_LRG', PNG),
  Image('BMP_TN', BMP),
  Image('BMP_LRG', BMP),

  Audio('MP3', MP3),
  Audio('LPCM', LPCM),
  Audio('FLAC', FLAC),
  Audio('WMABASE', WMA),
  Audio('WMAFULL', WMA),
  Audio('WMAPRO', WMA),
  Audio('WAV', WAV),

  Video('MPEG_PS_PAL', MPEG),
  Video('MPEG_PS_NTSC', MPEG),
  Video('MPEG_TS_SD_EU_T', MTTS),
  Video('MPEG_TS_SD_EU_ISO', MTTS),
  Video('MPEG_TS_SD_EU', MTTS),
  Video('MPEG_TS_SD_NA_T', MTTS),
  Video('MPEG_TS_SD_NA_ISO', MTTS),
  Video('MPEG_TS_SD_NA', MTTS),
  Video('WMVSPLL_BASE', WMV),
  Video('WMVSPML_BASE', WMV),
  Video('WMVMED_BASE', WMV),
  Video('WMVMED_FULL', WMV),
  Video('WMVMED_PRO', WMV),
  Video('WMVHIGH_FULL', WMV),
  Video('WMVHIGH_PRO', WMV)

];

//-----------------------------------------------------------------------------

function onDlnaSource(sourceArray) {
  for (var i = 0; i < all.length; i++) {
    sourceArray.push(all[i].toString());
  }
}

evt.onDlnaSource(onDlnaSource);