/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var util = require('util');
var path = require('path');

var evt = require('../evt');
var log = require('../log');
var rti = require('../rti');
var cfg = require('../cfg');
var md5 = require('../md5');

var RootDevice = require('../cls/rootdevice').RootDevice;
var Icon = require('../cls/icon').Icon;

var id = rti.did;

function MediaServer() {

  RootDevice.call(this, 'dms1');

  var self = this;

  self.icon = new Icon(
    path.join(global['ISEDORA_DIR'], 'pub', 'ico'),
    'image/jpeg',
    '.jpg',
    [48, 64, 120]
//    [16, 32, 64, 120]
  );

  self[id.deviceType]       = 'urn:schemas-upnp-org:device:MediaServer:1';
  self[id.manufacturer]     = 'iSedora.com';
  self[id.manufacturerURL]  = 'http://www.isedora.com';
  self[id.modelName]        = 'iSedora';
  self[id.modelDescription] = 'Universal DLNA/UPnP Media Server';
  self[id.modelNumber]      = '02';
  self[id.modelURL]         = 'http://www.isedora.com';

  self.onDocReady = function(doc) {
    doc.device['#list'] = [
      {'dlna:X_DLNADOC': 'DMS-1.50'},
      {'dlna:X_DLNADOC': 'M-DMS-1.50'}
    ];
  };

  function onConfigReady() {
    self[id.UDN] = 'uuid:' + md5.uuid('iSedoraDMS1' + cfg.uuid());
    self[id.friendlyName] = cfg.serverName();
  }

  function onConfigChange() {
    self[id.friendlyName] = cfg.serverName();
  }

  evt.onConfigReady(onConfigReady);
  evt.onConfigChange(onConfigChange);

}

util.inherits(MediaServer, RootDevice);

var srv = new MediaServer();

exports.srv = srv;

evt.onGetServers(function(ary) {
  ary.push(srv);
});
