/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// File Operations

'use strict';

var fs    = require('fs');
var path  = require('path');
var mime  = require('mime');
var async = require('async');

var md5  = require('./md5');
var env  = require('./env');

var iswin = env.isWin();

exports.dummyCount = function(folderPath, cb) {
  cb(null, 10000);
};

exports.childCount = function(folderPath, cb) {
  fs.readdir(folderPath, function(err, names) {
    if (err) {
      cb(null, 0);
    }
    else {
      var len = names.length;
      if (iswin || len === 0) {
        return cb(null, len);
      }
      var cnt = len;
      for (var i = 0; i < len; i++) {
        if (names[i].charAt(0) === '.') cnt--;
      }
      cb(null, cnt);
    }
  });
};

exports.getFileName = function(name) {
  var pos = name.lastIndexOf(path.sep);
  if (pos > -1) name = name.substring(pos + 1);
  pos = name.lastIndexOf('.');
  return (pos > 0) ? name.substring(0, pos) : name;
};

exports.folderSizeFast = function(dirPath, cb) {

  var count = 0;
  var size  = 0;

  function on_each_fname(fname, xcb) {
    fs.stat(path.join(dirPath, fname), function(err, stat) {
      if (err) {
        return xcb(err);
      }
      if (stat.isFile()) {
        count++;
        size += stat.size;
      }
      xcb();
    });
  }

  fs.readdir(dirPath, function(err, fnames) {

    if (err) {
      return cb(err);
    }

    async.eachLimit(fnames, 4, on_each_fname, function(err) {
      if (err) {
        cb(err);
      }
      else {
        cb(null, {count: count, size: size});
      }
    });

  });

};

exports.deleteFiles = function(dirPath, cb) {

  function on_each_fname(fname, acb) {
    var filePath = path.join(dirPath, fname);
    fs.stat(filePath, function(err, stat) {
      if (err) {
        return acb(err);
      }
      if (stat.isFile()) {
        fs.unlink(filePath, function(err) {
          acb(err);
        });
      }
      else {
        acb();
      }
    });
  }

  fs.readdir(dirPath, function(err, fnames) {

    if (err) {
      return cb(err);
    }

    async.eachLimit(fnames, 2, on_each_fname, function(err) {
      cb(err);
    });

  });

};

//-----------------------------------------------------------------------------

exports.listDir = function(dir, cback) {

  fs.readdir(dir, function(err, files) {
    if (err) {
      cback(err);
    }
    else {
      var len = files.length;
      for (var i = 0; i < len; i++) {
        files[i] = path.join(dir, files[i]);
      }
      cback(null, files);
    }
  });

};

exports.fileSize = function(filepath, cback) {

  fs.stat(filepath, function(err, stat) {
    if (err) {
      cback(err);
    }
    else {
      cback(null, stat.size || 0);
    }
  });

};

exports.lastModified = function(filepath, cb) {

  fs.stat(filepath, function(err, stat) {
    if (err) {
      cb(err);
    }
    else {
      var mtime = stat.mtime;
      cb(null, (mtime ? mtime.getTime() : 0));
    }
  });

};