/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var express = require('express');
var swig = require('swig');
var path = require('path');
var fs = require('fs');
var evt = require('./evt');
var msg = require('./msg');
var log = require('./log');
var cfg = require('./cfg');
var ifa = require('./ifa');
var mod = require('./mod');
var dev = require('./ssdp/dev');
var upnp = require('./cls/upnp');

var mdb = require('./dlna/mdb');
var aboutData = require('./rest/about').data;
var srv = require('./dms1/media_server_1').srv;

//-----------------------------------------------------------------------------

function modPref(req, res) {
  var name = req.param('name');
  var pkg = mod.getPackage(name);
  if (!pkg) {
    return res.sendStatus(404);
  }
  res.render('prefs/plugpref', {
    cfg: cfg.getConfig(),
    mdb: mdb,
    sys: cfg.sysinfo(),
    ifa: ifa,
    dev: dev,
    srv: srv,
    mod: pkg,
    pkg: pkg,
    plugin: mod.getPlugin(name),
    pcfg: cfg.getPluginConfig(name),
    upnp: upnp,
    isLocal: ifa.hasAddr(req.ip)
  });
}

//-----------------------------------------------------------------------------

function devConf(req, res) {
  var uuid = req.params['uuid'];
  if (!uuid) {
    return res.sendStatus(404);
  }
  var dcfg = cfg.getDevCfg(uuid);
  if (!dcfg) {
    return res.sendStatus(404);
  }
  res.render('dconf', {
    uuid: uuid,
    dcfg: dcfg
  });
}

//-----------------------------------------------------------------------------

function prefPage(req, res) {
  var page = req.params[0];
  res.render('prefs/' + page, {
    cfg: cfg.getConfig(),
    mdb: mdb,
    sys: cfg.sysinfo(),
    ifa: ifa,
    dev: dev,
    srv: srv,
    plugins: mod.allPackages(),
    upnp: upnp,
    isLocal: ifa.hasAddr(req.ip)
  });
}

//-----------------------------------------------------------------------------

function trmap(req, res) {
  res.set('Content-Type', 'text/javascript; charset=utf-8');
  res.send('var trmap = ' + JSON.stringify(msg.all()) + ';');
}

//-----------------------------------------------------------------------------

function rtiScript(req, res) {
  res.set('Content-Type', 'text/javascript; charset=utf-8');
  res.sendFile(__dirname + '/rti.js');
}

//-----------------------------------------------------------------------------

function swigPage(req, res) {
  var page = req.params[0];
  if (page === undefined) {
    page = 'index';
  }
  res.render(page);
}

//-----------------------------------------------------------------------------

function forbidden(req, res) {
  res.status(403).send('Why do you want to see it? Forbidden.');
}

//-----------------------------------------------------------------------------

function initGUI(app) {

  app.engine('html', swig.renderFile);

  app.set('view engine', 'html');
  app.set('views', ISEDORA_DIR + '/gui');
  app.set('view cache', false);

  swig.setDefaults({
    cache: 'memory',
    locals: {
      tr: msg,
      about: aboutData,
      path:  cfg.facePath(),
      rest:  cfg.restPath(),
      dlna:  cfg.dlnaPath()
    }
  });

  var face = cfg.face(app);

  face.get('/js/trmap.js', trmap);
  face.get('/js/rti.js', rtiScript);

  face.get('/dconf/:uuid', devConf);

  face.get('/prefs/plugpref/:name', modPref);

  face.get('/prefs/*.html', prefPage);

  face.get('/', swigPage);
  face.get('/*.html', swigPage);

  face.get('/*.swig', forbidden);

  face.use(express.static(ISEDORA_DIR + '/gui'));

}

evt.onHttpAppReady(initGUI);


