/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// X-AV-Physical-Unit-Info: pa="Blu-ray Disc Player";
// X-AV-Client-Info: av=5.0; cn="Sony Corporation"; mn="Blu-ray Disc Player"; mv="2.0";

var XAV_UNIT_INFO   = 'X-AV-PHYSICAL-UNIT-INFO';
var XAV_CLIENT_INFO = 'X-AV-CLIENT-INFO';

var XML_MIME = 'text/xml; charset=UTF-8';
var TXT_MIME = 'text/plain; charset=UTF-8';

exports.XML_MIME = XML_MIME;
exports.TXT_MIME = TXT_MIME;

var GET_CONTENT_FEATURES_DLNA_ORG = "getcontentFeatures.dlna.org";
var CONTENT_FEATURES_DLNA_ORG     = "contentFeatures.dlna.org";
var TRANSFER_MODE = 'TransferMode.DLNA.ORG';
var TRANSFER_INTERACTIVE = 'Interactive';
var TRANSFER_STREAMING = 'Streaming';

exports.TRANSFER_INTERACTIVE = TRANSFER_INTERACTIVE;
exports.TRANSFER_STREAMING = TRANSFER_STREAMING;

exports.SEND_FILE_OPT = { etag: false };

exports.setXmlMime = function(res) {
  res.set('Content-Type', XML_MIME);
};

exports.setTextMime = function(res) {
  res.set('Content-Type', TXT_MIME);
};

function parseXAV(header) {
  var map = {};
  var ary = header.split(';');
  for (var i = 0; i < ary.length; i++) {
    var kv = ary[i].trim().split('=');
    if (kv.length === 2) {
      var key = kv[0].trim();
      var val = kv[1].trim();
      val = val.replace(/"/g, '');
      if (key.length && val.length) {
        map[key] = val;
      }
    }
  }
  return map;
}

exports.getXAV = function(req) {
  var unitInfo = req.get(XAV_UNIT_INFO);
  var clntInfo = req.get(XAV_CLIENT_INFO);
  if (!unitInfo && !clntInfo) {
    return null;
  }
  var friendlyName;
  var manufacturer;
  if (unitInfo) {
    var umap = parseXAV(unitInfo);
    friendlyName = umap['pa'];
  }
  if (clntInfo) {
    var cmap = parseXAV(clntInfo);
    if (!friendlyName) {
      friendlyName = cmap['mn'];
    }
    manufacturer = cmap['cn'];
  }
  if (friendlyName || manufacturer) {
    var hmap = {};
    if (friendlyName) {
      hmap.friendlyName = friendlyName;
    }
    if (manufacturer) {
      hmap.manufacturer = manufacturer;
    }
    return hmap;
  }
  return null;
};

exports.httpAddress = function(req) {
  var addr = req.socket.address();
  return 'http://' + addr.address + ':' + addr.port;
};

//-----------------------------------------------------------------------------
// Transfer Mode
//-----------------------------------------------------------------------------

function setTransferMode(res, mode) {
  res.set('TransferMode.DLNA.ORG', mode);
}

exports.setTransferMode = setTransferMode;

function setDlnaHeaders(srq, transferMode) {
  var req = srq.req;
  var res = srq.res;
  if (transferMode) {
    res.set(TRANSFER_MODE, transferMode);
  }
  if (req.get(GET_CONTENT_FEATURES_DLNA_ORG)) {
    res.set(CONTENT_FEATURES_DLNA_ORG, srq.prf);
  }
}

exports.setDlnaHeaders = setDlnaHeaders;

