/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var fs   = require('fs');
var path = require('path');
var util = require('util');
var log  = require('./log');
var evt  = require('./evt');
var pkg  = require(ISEDORA_PKG);

var DEF_LANG   = 'en';
var MSG_EXT    = ".msg";
var FREAD_OPTS = { encoding: 'utf8' };
var MSG_DIR    = path.join(ISEDORA_DIR, pkg['msgDir']);

var map = {};
var file = null;
var watcher = null;
var stamp = new Date();

var msg = function(key) {

  var val = map[key];

  if (val === undefined) {
    val = key;
  }

  if (arguments.length === 1) {
    return val;
  }

  arguments[0] = val;

  return util.format.apply(null, arguments);

};

function parseLine(target, line, len) {

  var pos = 0;

  for (;;) {

    pos = line.indexOf('=', pos);

    if (pos === -1 || pos === 0) return;

    if (line.charAt(pos - 1) === '\\') {
      pos++;
      continue;
    }

    var key = line.substring(0, pos).trim();
    var val = ((pos + 1) < len) ? line.substring(pos + 1).trim() : "";

    if (key.length > 0) {
      if (val.length == 0) {
        val = key;
      }
      target[key] = val;
    }

    break;

  }
}

function parseFile(fdata) {

  var target = {};

  var lines = fdata.split('\n');

  var count = lines.length;
  var line, i, char, len;

  for (i = 0; i < count; i++) {
    line = lines[i].trim();
    len = line.length;
    if (len > 2) {
      char = line.charAt(0);
      if (char !== '[' && char !== ';') {
        parseLine(target, line, len);
      }
    }
  }

  map = target;

}

function reloadMessages(event, filename) {
  log.debug('Reloading messages (%s): %s', event, file);
  fs.readFile(file, FREAD_OPTS, function(err, data) {
    if (err) {
      log.error('Error reading messages: %s', err);
    }
    else {
      stamp = new Date();
      parseFile(data);
    }
    watcher.close();
    watcher = fs.watch(file, { persistent: false });
    watcher.once('change', reloadMessages);
  });
}

msg.load = function(lang) {

  file = path.join(MSG_DIR, lang + MSG_EXT);

  if (!fs.existsSync(file)) {

    log.warn("Message file not found for: %s", lang);

    if (lang.indexOf('-') > 0) {
      msg.load(lang.split('-')[0]);
      return;
    }

    if (lang != DEF_LANG) {
      msg.load(DEF_LANG);
    }

    return;

  }

  log.debug("Loading messages from: %s", file);

  var fdata = fs.readFileSync(file, FREAD_OPTS);

  parseFile(fdata);

  stamp = new Date();

  watcher = fs.watch(file, { persistent: false });

  watcher.once('change', reloadMessages);

};

msg.all = function() {
  return map;
};

msg.stamp = function() {
  return stamp;
};

module.exports = msg;