/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Data Runtime Type Information

(function(exports) {

  // Media Object Properties

  var id = {
    uid:   'uid',   // Unique Resource ID
    error: 'error', // File processing error info
    fpath: 'fpath', // Full file/folder path
    flink: 'flink', // Real file/folder path for links
    fname: 'fname', // File/folder name without extension
    fextn: 'fextn', // File extension, not used for folders
    ftype: 'ftype', // File type (see below)
    fmode: 'fmode', // File/folder FS attributes
    fmime: 'fmime', // File mime type
    fsize: 'fsize', // File size
    froot: 'froot', // Top folder path
    mtime: 'mtime', // File/folder modification time in millis
    ctime: 'ctime', // File/folder creation time in millis
    mtype: 'mtype', // File media type (see below)
    index: 'index', // Curent image or PDF index
    fhidn: 'fhidn', // File/folder hidden
    // Image Related
    orn:    'orn',    // Orientation
    bpp:    'bpp',    // Bits per Pixel
    clr:    'clr',    // Color Model
    idxclr: 'idxclr', // Indexed color model
    width:  'width',  // Width
    height: 'height', // Height
    count:  'count',  // Number of sub images or PDF pages
    // Thumb Related
    tvalid:  'tvalid',  // Has valid thumbnail
    tpath:   'tpath',   // alternative thumb path
    twidth:  'twidth',  // Thumb width
    theight: 'theight',  // Thumb height
    // Meta Data
    title: 'title',
    // Top Folder
    istop: 'istop',
    // Lack modified check stamp
    stamp: 'stamp'
  };

  exports.id = id;

  // File Type

  var ftype = {
    unknown: 0,
    file:    1,
    folder:  2
  };

  exports.ftype = ftype;

  // Media Type

  var mtype = {
    unknown: 0,
    image:   1,
    audio:   2,
    video:   4,
    pdf:     8
  };

  exports.mtype = mtype;

  // Device Properties

  exports.did = {
    addr: 'addr',
    uuid: 'uuid',
    hash: 'hash',
    face: 'face',
    from: 'from',
    href: 'href',   // SSDP Location
    hmap: 'hmap',   // HTTP headers
    umap: 'umap',   // USN map (updated with notify)
    smap: 'smap',   // Services map (from description)
    alias: 'alias', // Device Name Alias
    ready: 'ready', // Device XML Loaded
    icons: 'icons', // icons map
    ctime: 'ctime', // Created
    mtime: 'mtime', // Updated / Modified
    atime: 'atime', // Last access time from device
    friendlyName: 'friendlyName',
    manufacturer: 'manufacturer',
    modelName: 'modelName',
    modelDescription: 'modelDescription',
    modelNumber: 'modelNumber',
    modelURL: 'modelURL',
    manufacturerURL: 'manufacturerURL',
    deviceType: 'deviceType',
    UDN: 'UDN',
    path: 'path',
    serviceType: 'serviceType',
    // Device Config
    dcfg:  'dcfg',
    title: 'title',
    sreso: 'sreso'
  };

  // Folder sort

  var sort = {
    name:   0,
    newest: 1,
    oldest: 2
  };

  exports.sort = sort;

  // Type Detectors

  exports.isFile = function(md) {
    return md[id.ftype] === ftype.file;
  };

  exports.isFolder = function(md) {
    return md[id.ftype] === ftype.folder;
  };

  exports.isUnknownFileType = function(md) {
    return md[id.ftype] === ftype.unknown;
  };

  exports.isHidden = function(md) {
    return md[id.fhidn];
  };

  exports.isUnknownMediaType = function(md) {
    return md[id.mtype] === mtype.unknown;
  };

  exports.isImage = function(md) {
    return md[id.mtype] === mtype.image;
  };

  exports.isAudio = function(md) {
    return md[id.mtype] === mtype.audio;
  };

  exports.isVideo = function(md) {
    return md[id.mtype] === mtype.video;
  };

  exports.isPDF = function(md) {
    return md[id.mtype] === mtype.pdf;
  };

  exports.isImageOrPDF = function(md) {
    var mt = md[id.mtype];
    return (mt === mtype.image || mt === mtype.pdf);
  };

  // Media Data Helpers

  exports.uid = function(md) {
    return md[id.uid];
  };

  exports.fname = function(md) {
    return md[id.fname];
  };

  exports.fpath = function(md) {
    return md[id.flink] || md[id.fpath];
  };

  // Thumbnail

  exports.isThumbValid = function(md) {
    return md[id.tvalid];
  };

  exports.thumbWidth = function(md) {
    return md[id.twidth];
  };

  exports.thumbHeight = function(md) {
    return md[id.theight];
  };

  // Generic Type Helpers

  exports.notNull = function(val) {
    return (val !== undefined && val !== null);
  };

  exports.isObject = function(arg) {
    return (typeof arg === 'object' && arg !== null && !Array.isArray(arg));
  };

  exports.isArray = function(arg) {
    return Array.isArray(arg);
  };

  exports.isFunction = function(arg) {
    return typeof arg === 'function';
  };

  exports.isNumber = function(arg) {
    return (typeof arg === 'number' && !isNaN(arg));
  };

  function objectToString(obj) {
    return Object.prototype.toString.call(obj);
  }

})(typeof exports === 'undefined'? this['rti'] = {} : exports);