/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var async = require('async');

var ssdp = require('../cls/ssdp');
var upnp = require('../cls/upnp');

var log = require('../log');
var evt = require('../evt');
var cfg = require('../cfg');
var acl = require('../acl');

var SSDP_ALL = 'ssdp:all';

var interval;
var nfaces = {};

//-----------------------------------------------------------------------------
// Search
//-----------------------------------------------------------------------------

function search() {
  for (var addr in nfaces) {
    searchFace(nfaces[addr]);
  }
}

function searchFace(face, cb) {
  if (!cfg.nicBlocked(face.name)) {
    log.ssdp('Searching on: %s', face.addr);
    ssdp.search(face, acl.checkSSDP, upnp.ROOT_DEVICE, 3, cfg.udpTTL(), cb);
  }
  else {
    if (cb) cb();
  }
}

exports.search = function(cb) {
  var vals = [];
  for (var addr in nfaces) {
    vals.push(nfaces[addr]);
  }
  async.each(vals, searchFace, cb);
};

//-----------------------------------------------------------------------------
// Interval
//-----------------------------------------------------------------------------

function startInterval() {
  if (!interval) {
    interval = setInterval(search, cfg.searchInterval() * 1000);
  }
}

function resetInterval() {
  if (interval) {
    clearInterval(interval);
    interval = null;
  }
}

//-----------------------------------------------------------------------------
// NetFace Events
//-----------------------------------------------------------------------------

function nicReady(face) {
  nfaces[face.addr] = face;
  setTimeout(function() {
    searchFace(face);
  }, 100);
  startInterval();
}

function nicDone(face) {
  delete nfaces[face.addr];
}

evt.onNetFaceOn(nicReady);
evt.onNetFaceOff(nicDone);

//-----------------------------------------------------------------------------
// Event Hooks
//-----------------------------------------------------------------------------

evt.onConfigChange(function() {
  resetInterval();
  startInterval();
  setImmediate(search);
});

evt.onExit(function() {
  resetInterval();
});