/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var ssdp = require('../cls/ssdp');

var log = require('../log');
var evt = require('../evt');
var cfg = require('../cfg');
var acl = require('../acl');
var fmt = require('../fmt');

var lsmap = {}; // Listeners map

//-----------------------------------------------------------------------------
// Message Logger
//-----------------------------------------------------------------------------

function msgLogger(msg) {
  if (log.canSsdp()) {
    if (msg.isNotify()) {
      log.ssdp('%s from %s %s', msg.nts, msg.from.addr, msg.nt);
    }
    else if (msg.isSearch()) {
      log.ssdp('Search request from %s %s', msg.from.addr, msg.st);
    }
    else if (msg.isHttp11()) {
      log.ssdp('Search response from %s %s', msg.from.addr, msg.usn);
    }
    else {
      log.ssdp('Message from %s\n%s', msg.from.addr, log.border(msg.text));
    }
  }
}

//-----------------------------------------------------------------------------
// NetFace Events
//-----------------------------------------------------------------------------

function nicReady(face) {

  var ls = lsmap[face.addr];

  if (ls && ls.isok()) {
    return;
  }

  ls = new ssdp.Listener(face, acl.checkSSDP);

  ls.init();

  lsmap[face.addr] = ls;

}

function nicDone(face) {
  var lst = lsmap[face.addr];
  if (lst) {
    delete lsmap[face.addr];
    lst.done();
  }
}

function exitProc(cb) {
  for (var lst in lsmap) {
    lsmap[lst].done();
  }
  cb();
}

evt.addExitProc(exitProc);
evt.onNetFaceOn(nicReady);
evt.onNetFaceOff(nicDone);

evt.onSsdpMessage(msgLogger);


