/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// PDF Viewer Plugin

'use strict';

var async = require('async');
var child = require('child_process');

var log = isedora['log'];
var evt = isedora['evt'];
var env = isedora['env'];
var cfg = isedora['cfg'];
var hnd = isedora['hnd'];
var rti = isedora['rti'];
var htp = isedora['htp'];
var mdb = isedora['dlna']['mdb'];
var mdi = isedora['dlna']['mdi'];
var alg = isedora['dlna']['alg'];
var prf = isedora['dlna']['prf'];
var tmb = isedora['dlna']['tmb'];

var didl = isedora['cls']['didl'];
var upnp = didl.upnp;

var minfo = mdb.minfo;

var id = rti.id;
var mtype = rti.mtype;

var Container = didl.Container;
var Res = didl.Res;

var PLUGIN_NAME = 'isedora-pdf';
var HANDLE_NAME = 'pdf';

var enabled = isEnabled();

//-----------------------------------------------------------------------------

function pdfBrowser(brq, cb) {
  var uid = brq.uid;
  var doc = (uid.charAt(0) === 'd');
  uid = uid.substring(1);
  var md = minfo.get(uid);
  if (!md) {
    return cb('PDF document is not in media cache: ' + brq.oid);
  }
  if (brq.meta) {
    if (doc) {
      brq.append(pdfContainer(brq, md));
    }
    else {
      brq.append(pdfPage(brq, md));
    }
    return cb();
  }
  var start = brq.start;
  var count = brq.count;
  var total = md[id.count];
  brq.range = true;
  brq.total = total;
  if (start >= total) return cb();
  var pages = [];
  for (var i = start; i < (start + count) && i < total; i++) {
    pages.push(i);
  }
  var fpath = rti.fpath(md);
  async.map(pages,
    function(pnum, pcb) {
      var pid = uid;
      if (pnum > 0) pid += '_' + pnum;
      var pmd = minfo.get(pid);
      if (pmd) {
        return pcb(null, pdfPage(brq, pmd));
      }
      mdi.fileInfo(fpath, pnum, true, function(err, pmd) {
        if (!rti.isPDF(pmd)) {
          brq.total--;
          return pcb(null, null);
        }
        pcb(null, pdfPage(brq, pmd));
      });
    },
    function(err, pdidl) {
      brq.addall(pdidl);
      cb();
    }
  );
}

hnd.addBrowseHandler(HANDLE_NAME, pdfBrowser);

//-----------------------------------------------------------------------------

function pdfPage(brq, md) {
  var uid = rti.uid(md);
  var tid = hnd.fsoid(mtype.image, uid);
  var pid = mkoid(uid, false);
  var page = didl.imageItem(pid, brq.oid, 'Page ' + (md[id.index] + 1));
  page.setDate(md[id.mtime]);
  var wh = pdfWH(brq.dev, md);
  var ri = new Res(hnd.streamURL(brq.furl, pid, prf.JPEG_LRG), prf.JPEG_LRG);
  ri.setColorDepth(24);
  ri.setResolution(wh[0], wh[1]);
  page.addResource(ri);
  if (md[id.tvalid]) {
    var rt = new Res(hnd.streamURL(brq.turl, tid, prf.JPEG_TN), prf.JPEG_TN);
    rt.setColorDepth(24);
    rt.setResolution(md[id.twidth], md[id.theight]);
    page.addResource(rt);
  }
  return page;
}

function pdfContainer(brq, md) {
  var pdf = new Container(mkoid(rti.uid(md), true), brq.oid, rti.fname(md) + '.pdf', md[id.count]);
  pdf.setMultiItem(true);
  return pdf;
}

function pdfMedia(brq, md, cb) {
  if (!enabled || !rti.isPDF(md)) return;
  cb(null, pdfContainer(brq, md));
  return true;
}

hnd.addMediaHandler(pdfMedia);

//-----------------------------------------------------------------------------
// Stream Handler
//-----------------------------------------------------------------------------

function pdfStreamHandler(srq) {
  if (srq.hnd !== HANDLE_NAME) return;
  var uid = srq.uid.substring(1);
  var md = minfo.get(uid);
  if (!md) return;
  var wh = pdfWH(srq.dev, md);
  htp.setDlnaHeaders(srq, htp.TRANSFER_INTERACTIVE);
  tmb.sendScale(md, wh[0], wh[1], srq.res);
  return true;
}

hnd.addStreamHandler(pdfStreamHandler);

//-----------------------------------------------------------------------------

function pdfWH(dev, md) {
  var w = md[id.width];
  var h = md[id.height];
  var sr = dev.sreso;
  return alg.scale(w, h, sr[0], sr[1]);
}

function mkoid(uid, doc) {
  if (doc) {
    uid = 'd' + uid;
  }
  else {
    uid = 'p' + uid;
  }
  return hnd.mkoid(HANDLE_NAME, uid);
}

//-----------------------------------------------------------------------------

function onConfigChange() {
  enabled = isEnabled();
}

evt.onConfigChange(onConfigChange);

//-----------------------------------------------------------------------------

function isEnabled() {
  return cfg.pluginEnabled(PLUGIN_NAME);
}

//-----------------------------------------------------------------------------

if (!cfg.getPluginConfig(PLUGIN_NAME)) {
  cfg.setPluginConfig(PLUGIN_NAME, {});
}
